package fi.aalto.data.server;

import java.util.Locale;

import com.prosysopc.ua.stack.builtintypes.LocalizedText;
import com.prosysopc.ua.stack.builtintypes.NodeId;
import com.prosysopc.ua.stack.core.Identifiers;

import com.prosysopc.ua.StatusException;
import com.prosysopc.ua.nodes.UaNodeFactoryException;
import com.prosysopc.ua.nodes.UaNode;
import com.prosysopc.ua.server.NodeManagerUaNode;
import com.prosysopc.ua.server.UaInstantiationException;
import com.prosysopc.ua.server.UaServer;
import com.prosysopc.ua.server.nodes.PlainVariable;
import com.prosysopc.ua.server.nodes.UaObjectNode;

public class DataNodeManager extends NodeManagerUaNode {

	public static final String NAMESPACE = "http://localhost/OPCUA/DataAddressSpace";

	public DataNodeManager(UaServer arg0, String arg1) {
		super(arg0, arg1);
		// Auto-generated constructor stub
	}

	private void createAddressSpace() throws StatusException, UaInstantiationException {
		// Create data address space

		int ns = getNamespaceIndex();
		final UaNode objectsFolder = getServer().getNodeManagerRoot().getObjectsFolder();

		UaNode parent = objectsFolder;

		// Base objects
		parent = createFolder(ns, "CoDeSys.OPC.DA", parent);
		parent = createFolder(ns, "PLC_GW3", parent);
		parent = createFolder(ns, "Application", parent);
		parent = createFolder(ns, "GVL_WP_HPP", parent);
				
        // L300 High level alarm
		createVariable(ns, "L300_MeasVal", false, Identifiers.Boolean, parent);

		// L301 Low level alarm
		createVariable(ns, "L301_MeasVal", false, Identifiers.Boolean, parent);

		// P300 Pressure sensor
		createVariable(ns, "P300_MeasVal", new Float(0), Identifiers.Float, parent);
		
		// T300 Temperature sensor
		createVariable(ns, "T300_MeasVal", new Float(0), Identifiers.Float, parent);
		
		// M200 Pump
		createVariable(ns, "M200_CtrlOut", new Integer(0), Identifiers.Integer, parent);
		createVariable(ns, "M200_CurCtrlVal", new Float(0), Identifiers.Float, parent);
		
		// Y301 and Y303 Cut-off valves
		createVariable(ns, "Y301_CtrlOut", false, Identifiers.Boolean, parent);
		createVariable(ns, "Y301_CurOutINT", new Integer(0), Identifiers.Integer, parent);
		createVariable(ns, "Y301_CtrlVal", false, Identifiers.Boolean, parent);

		createVariable(ns, "Y303_CtrlOut", false, Identifiers.Boolean, parent);
		createVariable(ns, "Y303_CurOutINT", new Integer(0), Identifiers.Integer, parent);
		createVariable(ns, "Y303_CtrlVal", false, Identifiers.Boolean, parent);
		
		// Y501 Control valve
		createVariable(ns, "Y501_CtrlOut", new Integer(0), Identifiers.Integer, parent);
		createVariable(ns, "Y501_CtrlVal", new Float(0), Identifiers.Float, parent);
		createVariable(ns, "Y501_CurCtrlVal", new Float(0), Identifiers.Float, parent);
		
		return;
	}	

	@Override
	protected void init() throws StatusException, UaNodeFactoryException {
		super.init();

		createAddressSpace();
	}

	private UaObjectNode createFolder(int ns, String name, UaNode parent)
	{
	    final NodeId id = new NodeId(ns, name);
	    UaObjectNode node = new UaObjectNode(this, id, name, Locale.ENGLISH);
	    node.setTypeDefinitionId(Identifiers.FolderType);
		try {
		    this.addNodeAndReference(parent, node, Identifiers.Organizes);
		} catch (Exception e) {
			System.out.println(e.getMessage());
		}
		node.setDescription(new LocalizedText(name)); 
		
		return node;
	}

	private <T> PlainVariable<T> createVariable(int ns, String name, T value, NodeId dataTypeId, UaNode parent)
	{
		NodeId id = new NodeId(ns, name);
		PlainVariable<T> variable = new PlainVariable<T>(this, id, name, LocalizedText.NO_LOCALE);
		variable.setDataTypeId(dataTypeId);
		variable.setTypeDefinitionId(Identifiers.BaseDataVariableType);
		parent.addComponent(variable);
		variable.setCurrentValue(value);
		variable.setDescription(new LocalizedText(name)); 

		return variable;
	}
	
}
